unit TTAReader;
(*
 * TTAReader.cpp
 *
 * Description: TTA decompressor functions
 *
 * Copyright (c) 2004 Alexander Djourik. All rights reserved.
 * Copyright (c) 2004 Pavel Zhilin. All rights reserved.
 *
 *)

(*
 * TTAReader.cpp
 *
 * Description: TTA decompressor functions
 *
 * Copyright (c) 2004 Alexander Djourik. All rights reserved.
 * Copyright (c) 2004 Pavel Zhilin. All rights reserved.
 *
 *)

(*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * aint with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Please see the file COPYING in this directory for full copyright
 * information.
 *)

(* delphi transformation by K.Kumaki
 *   add TBitReader : SetFrame
 *   add TTTAReader : SetFrame , framesize
 *)

interface

uses Windows, Messages, SysUtils, Classes, TTAError, TTACommon, crc32;

type
  TBitReader = class(TObject)
  private
    hInFile: TStream;
    bit_buffer: array [0..BIT_BUFFER_SIZE+7] of byte;
    bit_buffer_end: pbyte;
    frame_crc32: ULONG;
    bit_count: ULONG;
    bit_cache: ULONG;
    bitpos: pbyte;
    st: PULONG;
    tta_head_pos, first_frame_pos, next_frame_pos: ULONG;
    finput_byte_count: ULong;
  protected
  public
    constructor Create(fd: TStream);
    destructor Destroy; override;
    procedure GetHeader(var ttahdr: TTTAHeader);
    function GetSeekTable(var seek_table: PULONG; st_size: LongInt): Boolean;
    procedure GetBinary(var value: ULONG; bits: ULONG);
    procedure GetUnary(var value: ULONG);
    function Done: Boolean;
    procedure SkipFrame;
    procedure SetFrame(fp: LongInt; seek_table: PULONG);
    property input_byte_count: ULONG read finput_byte_count write finput_byte_count;
  end;

  TTTAReader = class(TObject)
  private
    hInFile: TStream;
    data: PLongInt;
    offset, is_float, framelen, lastlen, framelen_, frame_size: ULONG;
    fframes, fframes_, byte_size, num_chan: ULONG;
    seek_table: PULONG;
    st_state: Boolean;
    tta, enc: Pencoder;
    bitReader: TBitReader;
    finput_byte_count: ULONG;
    foutput_byte_count: ULONG;
    fttahdr: TTTAHeader;
  protected
  public
    constructor Create(fd: TStream);
    destructor Destroy; override;
    function GetBlock(var buf: PLongInt): LongInt;
    procedure SetFrame(fp: LongInt);
    property input_byte_count: ULONG read finput_byte_count;
    property output_byte_count: ULONG read foutput_byte_count;
    property ttahdr: TTTAHeader read fttahdr;
    property framesize: ULONG read frame_size;
  end;

implementation

const
  WAVE_FORMAT_PCM = 1;
  WAVE_FORMAT_IEEE_FLOAT = 3;

{ TBitReader }

constructor TBitReader.Create(fd: TStream);
begin
  frame_crc32 := $FFFFFFFF;
  hInFile := fd;
  bit_count := 0;
  bit_cache := 0;
  bit_buffer_end := @bit_buffer[0];
  inc(bit_buffer_end, BIT_BUFFER_SIZE);
  bitpos := bit_buffer_end;
  finput_byte_count := 0;
end;

destructor TBitReader.Destroy;
begin
  inherited Destroy;
end;

procedure TBitReader.GetHeader(var ttahdr: TTTAHeader);
var
  checksum: ULONG;
begin
  tta_head_pos := hInFile.Position;
  if hInFile.Read(ttahdr, sizeof(TTTAHeader)) <> sizeof(TTTAHeader) then
    raise ETTA_READ_ERROR.Create('BitReader.GetHeader: Read Error')
  else inc(finput_byte_count, sizeof(TTTAHeader));
  
  // check for supported formats
  if ttahdr.TTAid <> TTA1_SIGN then
    raise ETTA_FORMAT_ERROR.Create('BitReader.GetHeader: Format Error');
  
  checksum := tta_crc32(pbyte(@ttahdr), sizeof(TTTAHeader) - sizeof(ULONG));
  if checksum <> ttahdr.CRC32 then
    raise ETTA_FILE_ERROR.Create('BitReader.GetHeader: File Error');
end;

function TBitReader.GetSeekTable(var seek_table: PULONG; st_size: LongInt): Boolean;
var
  checksum: ULONG;
  p: PULONG;
begin
  Result := false;
  if hInFile.Read(seek_table^, st_size * sizeof(ULONG)) <> st_size * sizeof(ULONG) then
    raise ETTA_READ_ERROR.Create('BitReader.GetSeekTable: Read Error')
  else inc(finput_byte_count, st_size * sizeof(ULONG));
  
  checksum := tta_crc32(pbyte(seek_table), (st_size - 1) * sizeof(ULONG));
  p := seek_table; inc(p, st_size - 1);
  if checksum = p^ then Result := true;

  next_frame_pos := hInFile.Position;
  first_frame_pos := next_frame_pos;
  st := seek_table;
end;

procedure TBitReader.GetBinary(var value: ULONG; bits: ULONG);
var
  rst: ULong;
begin
  while bit_count < bits do begin
    if bitpos = bit_buffer_end then begin
      rst := hInFile.Read(bit_buffer[0], BIT_BUFFER_SIZE);
      if rst = 0 then
        raise ETTA_READ_ERROR.Create('BitReader.GetBinary: Read Error');
      inc(finput_byte_count, rst);
      bitpos := @bit_buffer[0];
    end;
    
//  UPDATE_CRC32(bitpos^, frame_crc32);
    frame_crc32 := ((frame_crc32 shr 8) and $00FFFFFF) xor crc32_table[(frame_crc32 xor bitpos^) and $FF];
    bit_cache := bit_cache or (bitpos^ shl bit_count);
    inc(bit_count, 8);
    inc(bitpos);
  end;
  
  value := bit_cache and bit_mask[bits];
  bit_cache := bit_cache shr bits;
  dec(bit_count, bits);
  bit_cache := bit_cache and bit_mask[bit_count];
end;

procedure TBitReader.GetUnary(var value: ULONG);
var
  rst: ULONG;
begin
  value := 0;
  
  while (bit_cache xor bit_mask[bit_count]) = 0 do begin
    if bitpos = bit_buffer_end then begin
      rst := hInFile.Read(bit_buffer[0], BIT_BUFFER_SIZE);
      if rst = 0 then
        raise ETTA_READ_ERROR.Create('BitReader.GetUnary: Read Error');
      inc(finput_byte_count, rst);
      bitpos := @bit_buffer[0];
    end;
    
    inc(value, bit_count);
    bit_cache := bitpos^;
    inc(bitpos);
//  UPDATE_CRC32(bit_cache, frame_crc32);
    frame_crc32 := ((frame_crc32 shr 8) and $00FFFFFF) xor crc32_table[(frame_crc32 xor bit_cache) and $FF];
    bit_count := 8;
  end;
  
  while (bit_cache and 1) > 0 do begin
    inc(value);
    bit_cache := bit_cache shr 1;
    dec(bit_count);
  end;
  
  bit_cache := bit_cache shr 1;
  dec(bit_count);
end;

function TBitReader.Done: Boolean;
var
  crc32_, rbytes, rst: ULONG;
begin
  frame_crc32 := frame_crc32 xor $FFFFFFFF;
  
  inc(next_frame_pos, st^);
  inc(st);
  
  rbytes := ULONG(bit_buffer_end) - ULONG(bitpos);
  if rbytes < sizeof(ULONG) then begin
    move(bitpos^, bit_buffer[0], 4);
    rst := hInFile.Read(bit_buffer[rbytes], BIT_BUFFER_SIZE - rbytes);
    if rst = 0 then
      raise ETTA_READ_ERROR.Create('BitReader.Done: Read Error');
    inc(finput_byte_count, rst);
    bitpos := @bit_buffer[0];
  end;
  
  move(bitpos^, crc32_, 4);
  inc(bitpos, sizeof(ULONG));
  Result := (crc32_ <> frame_crc32);
  
  bit_cache := 0;
  bit_count := 0;
  frame_crc32 := $FFFFFFFF;
end;

procedure TBitReader.SkipFrame;
begin
  hInFile.Position := next_frame_pos;
  bitpos := bit_buffer_end;
end;

procedure TBitReader.SetFrame(fp: LongInt; seek_table: PULONG);
var
  i: LongInt;
begin
  next_frame_pos := first_frame_pos;
  st := seek_table;
  for i:=0 to fp-1 do begin
    inc(next_frame_pos, st^);
    inc(st);
  end;
  hInFile.Position := next_frame_pos;
  finput_byte_count := next_frame_pos - tta_head_pos;
  bitpos := bit_buffer_end;
end;

{ TTTAReader }

constructor TTTAReader.Create(fd: TStream);
var
  data_size: ULONG;
  st_size: Integer;
begin
  hInFile := fd;
  
  // clear statistics
  foutput_byte_count := 0;
  bitReader := TBitReader.Create(hInFile);
  bitReader.GetHeader(fttahdr);
  byte_size := (fttahdr.BitsPerSample + 7) div 8;
  framelen := Trunc(FRAME_TIME * fttahdr.SampleRate+1e-9);
  framelen_ := framelen;
  if fttahdr.AudioFormat = WAVE_FORMAT_IEEE_FLOAT then is_float := 1
  else is_float := 0;
  if is_float = 1 then // WAVE_FORMAT_IEEE_FLOAT no support
    raise ETTA_FORMAT_ERROR.Create('TTAReader.Create: Format Error');
  
  num_chan := fttahdr.NumChannels; // shl is_float;
  data_size := fttahdr.DataLength * byte_size * fttahdr.NumChannels;
  
  fframes := fttahdr.DataLength div framelen;
  lastlen := fttahdr.DataLength - fframes * framelen;
  if lastlen <> 0 then inc(fframes);
  st_size := (fframes + 1);
  st_state := False;
  frame_size := framelen * num_chan * byte_size;
  fframes_ := fframes;

  getmem(tta, sizeof(Tencoder) * num_chan);
  enc := tta;
  getmem(seek_table, sizeof(ULONG) * st_size);
  getmem(data, sizeof(LongInt) * framelen * num_chan);
  st_state := bitReader.GetSeekTable(seek_table, st_size);
  tta_encoder_init(tta, num_chan, byte_size);
end;

destructor TTTAReader.Destroy;
begin
  if Assigned(bitReader) then bitReader.Free;
  if Assigned(seek_table) then Freemem(seek_table);
  if Assigned(tta) then Freemem(tta);
  if Assigned(data) then Freemem(data);
  inherited Destroy;
end;

function TTTAReader.GetBlock(var buf: PLongInt): LongInt;
var
  p, q, r: PLongInt;
  value: LongInt;
  unary, binary, k: ULONG;
  i, j: integer;
  fst: Pfltst;
  rice: Padapt;
  last: PLongInt;
  pA, pB, pM: PLongInt;
  sum: LongInt;
  b: byte;
begin
  Result := 0;
  if fframes = 0 then exit;
  dec(fframes);
  
  if (fframes = 0) and (lastlen > 0) then framelen := lastlen;
  
  tta_encoder_init(tta, num_chan, byte_size);

  p := data;
  for i := 0 to framelen * num_chan - 1 do begin
    fst  := @enc.fst;
    rice := @enc.rice;
    last := @enc.last;
    
    // decode Rice unsigned
    bitReader.GetUnary(unary);
    
    if unary = 0 then begin
      k := rice.k0;
      if k > 0 then begin
        bitReader.GetBinary(binary, k);
        value := (unary shl k) + binary;
      end
      else value := unary;
    end
    else begin  // depth expand
      k := rice.k1; dec(unary);
      if k > 0 then begin
        bitReader.GetBinary(binary, k);
        value := (unary shl k) + binary;
      end
      else value := unary;
      rice.sum1 := rice.sum1 + value - (rice.sum1 shr 4);
      if (k > 0) and  (rice.sum1 < shift_16[k]) then
        dec(rice.k1)
      else if rice.sum1 > shift_16[k + 1] then
        inc(rice.k1);
      inc(value, bit_shift[rice.k0]);
    end;
    // default
    rice.sum0 := rice.sum0 + value - (rice.sum0 shr 4);
    if (rice.k0 > 0) and (rice.sum0 < shift_16[rice.k0]) then
      dec(rice.k0)
    else if rice.sum0 > shift_16[rice.k0 + 1] then
      inc(rice.k0);

//  p^ = DEC(value);
//    if (value and 1) <> 0 then p^ := (value + 1) shr 1
//    else p^ := -(value shr 1);
    if (value and 1) <> 0 then
    begin
      inc(value);
      asm  SAR value,1 end;
      p^ := value;
    end
    else begin
      asm  SAR value,1 end;
      p^ := -value;
    end;

    // decompress stage 1: adaptive hybrid filter
//  hybrid_filter(fst, p, 0);
    pA := @fst.dl[0];
    pB := @fst.qm[0];
    pM := @fst.dx[0];
    sum := fst.round;

    if fst.error = 0 then begin
      inc(sum, pA^ * pB^); inc(pA); inc(pB);
      inc(sum, pA^ * pB^); inc(pA); inc(pB);
      inc(sum, pA^ * pB^); inc(pA); inc(pB);
      inc(sum, pA^ * pB^); inc(pA); inc(pB);
      inc(sum, pA^ * pB^); inc(pA); inc(pB);
      inc(sum, pA^ * pB^); inc(pA); inc(pB);
      inc(sum, pA^ * pB^); inc(pA); inc(pB);
      inc(sum, pA^ * pB^); inc(pA); inc(pB); inc(pM, 8);
    end
    else if fst.error < 0 then begin
      dec(pB^, pM^); inc(sum, pA^ * pB^); inc(pA); inc(pB); inc(pM);
      dec(pB^, pM^); inc(sum, pA^ * pB^); inc(pA); inc(pB); inc(pM);
      dec(pB^, pM^); inc(sum, pA^ * pB^); inc(pA); inc(pB); inc(pM);
      dec(pB^, pM^); inc(sum, pA^ * pB^); inc(pA); inc(pB); inc(pM);
      dec(pB^, pM^); inc(sum, pA^ * pB^); inc(pA); inc(pB); inc(pM);
      dec(pB^, pM^); inc(sum, pA^ * pB^); inc(pA); inc(pB); inc(pM);
      dec(pB^, pM^); inc(sum, pA^ * pB^); inc(pA); inc(pB); inc(pM);
      dec(pB^, pM^); inc(sum, pA^ * pB^); inc(pA); inc(pB); inc(pM);
    end
    else begin
      inc(pB^, pM^); inc(sum, pA^ * pB^); inc(pA); inc(pB); inc(pM);
      inc(pB^, pM^); inc(sum, pA^ * pB^); inc(pA); inc(pB); inc(pM);
      inc(pB^, pM^); inc(sum, pA^ * pB^); inc(pA); inc(pB); inc(pM);
      inc(pB^, pM^); inc(sum, pA^ * pB^); inc(pA); inc(pB); inc(pM);
      inc(pB^, pM^); inc(sum, pA^ * pB^); inc(pA); inc(pB); inc(pM);
      inc(pB^, pM^); inc(sum, pA^ * pB^); inc(pA); inc(pB); inc(pM);
      inc(pB^, pM^); inc(sum, pA^ * pB^); inc(pA); inc(pB); inc(pM);
      inc(pB^, pM^); inc(sum, pA^ * pB^); inc(pA); inc(pB); inc(pM);
    end;

    q := pM; r := pA; dec(r);
(*  q^ := ((r^ shr 30) or 1) shl 2; dec(q); dec(r);
    q^ := ((r^ shr 30) or 1) shl 1; dec(q); dec(r);
    q^ := ((r^ shr 30) or 1) shl 1; dec(q); dec(r);
    q^ := ((r^ shr 30) or 1); *)

    value := r^;
    asm
      SAR     value,30
      OR      value,1
      SAL     value,2
    end;
    q^ := value;
    dec(q); dec(r);
    value := r^;
    asm
      SAR     value,30
      OR      value,1
      SAL     value,1
    end;
    q^ := value;
    dec(q); dec(r);
    value := r^;
    asm
      SAR     value,30
      OR      value,1
      SAL     value,1
    end;
    q^ := value;
    dec(q); dec(r);
    value := r^;
    asm
      SAR     value,30
      OR      value,1
    end;
    q^ := value;

//  if mode <> 0 then begin
//    pA^ := p^;
//    dec(p^, (sum shr fst.shift));
//    fst.error := p^;
//  end
//  else begin
      fst.error := p^;
//      inc(p^, (sum shr fst.shift));
      b := fst.shift;
      asm
        MOV CL,b
        SAR sum,CL
      end;
      inc(p^, sum);
      pA^ := p^;
//  end;

    if fst.mutex <> 0 then begin
      q := pA; r := q; dec(r);
      r^ := q^ - r^; dec(q); dec(r);
      r^ := q^ - r^; dec(q); dec(r);
      r^ := q^ - r^;
    end;

//  memshl (fs->dl, fs->dl + 1);
    q := @fst.dl[0]; r := q; inc(r);
    q^ := r^; inc(q); inc(r);
    q^ := r^; inc(q); inc(r);
    q^ := r^; inc(q); inc(r);
    q^ := r^; inc(q); inc(r);
    q^ := r^; inc(q); inc(r);
    q^ := r^; inc(q); inc(r);
    q^ := r^; inc(q); inc(r);
    q^ := r^;

//  memshl (fs->dx, fs->dx + 1);
    q := @fst.dx[0]; r := q; inc(r);
    q^ := r^; inc(q); inc(r);
    q^ := r^; inc(q); inc(r);
    q^ := r^; inc(q); inc(r);
    q^ := r^; inc(q); inc(r);
    q^ := r^; inc(q); inc(r);
    q^ := r^; inc(q); inc(r);
    q^ := r^; inc(q); inc(r);
    q^ := r^;
// end of hybrid_filter
    
    // decompress stage 2: fixed order 1 prediction
    case byte_size of
      1: p^ := p^ + LongInt(((uint64(last^) shl 4) - last^) shr 4); // bps 8
      2: p^ := p^ + LongInt(((uint64(last^) shl 5) - last^) shr 5); // bps 16
      3: p^ := p^ + LongInt(((uint64(last^) shl 5) - last^) shr 5); // bps 24
      4: p^ := p^ + last^; // bps 32
    end;
    last^ := p^;
    
    if ULONG(enc) < ULONG(tta) + (num_chan - 1) * sizeof(Tencoder) then inc(enc)
    else begin
      if num_chan > 1 then begin
        r := p; dec(r); q := p;
        p^ := p^ + (r^ div 2);
        for j:=1 to num_chan-1 do begin
          r^ := q^ - r^;
          dec(r); dec(q);
        end;
      end;
      enc := tta;
    end;
    inc(p);
  end;
  
  if bitReader.Done then begin // CRC error
    if st_state then begin
      bitReader.SkipFrame;
      FillChar(data^, num_chan * framelen * sizeof(LongInt), 0);
    end
    else
      raise ETTA_FILE_ERROR.Create('TTAWriter.GetBlock: File Error');
  end;
  
  buf := data;
  
  finput_byte_count := bitReader.input_byte_count;
  Result := (ULONG(p) - ULONG(data)) div sizeof(ULONG);
  foutput_byte_count := foutput_byte_count + Result * byte_size;
  
  Result := Result div num_chan;
end;

procedure TTTAReader.SetFrame(fp: LongInt);
begin
  framelen := framelen_;
  fframes := fframes_;
  if fp > fframes then fp := fframes;
  bitReader.SetFrame(fp, seek_table);
  dec(fframes, fp);
  finput_byte_count := bitReader.input_byte_count;
  foutput_byte_count := fp * frame_size;
end;

end.
