unit ttacommon;

{$X+}{$Z2}{$A1}

interface

uses Windows;

(*
 * ttacommon.h
 *
 * Description: TTA library common definitions and prototypes
 *
 * Copyright (c) 2004 Alexander Djourik. All rights reserved.
 * Copyright (c) 2004 Pavel Zhilin. All rights reserved.
 *
 *)

(*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * aint with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Please see the file COPYING in this directory for full copyright
 * information.
 *)

 (* delphi transformation by K.Kumaki *)

const
  MAX_BPS	= 32;
  FRAME_TIME    = 1.04489795918367346939;

  TTA1_SIGN	= $31415454;

  MAX_ORDER	= 16;
  BIT_BUFFER_SIZE	= $00100000;

type
  uint64	= Int64;

// basics structures definitions
  Padapt = ^Tadapt;
  Tadapt = record
	k0	: ULONG;
	k1	: ULONG;
	sum0	: ULONG;
	sum1	: ULONG;
  end;

  Pfltst = ^Tfltst;
  Tfltst = record
	shift	: LongInt;
	round	: LongInt;
	error	: LongInt;
	mutex	: LongInt;
	qm	: array[0..MAX_ORDER-1] of LongInt;
	dx	: array[0..MAX_ORDER-1] of LongInt;
	dl	: array[0..MAX_ORDER-1] of LongInt;
  end;

  Pencoder = ^Tencoder;
  Tencoder = record
	fst	: Tfltst;
	rice	: Tadapt;
	last	: LongInt;
  end;

  PTTAHeader = ^TTTAHeader;
  TTTAHeader = record
	TTAid	: ULONG;
	AudioFormat	: WORD;
	NumChannels	: WORD;
	BitsPerSample	: WORD;
	SampleRate	: ULONG;
	DataLength	: ULONG;
	CRC32	: ULONG;
  end;

const
// ****************** static variables and structures *******************
  bit_mask : array[0..32] of ULONG = (
    $00000000, $00000001, $00000003, $00000007,
    $0000000f, $0000001f, $0000003f, $0000007f,
    $000000ff, $000001ff, $000003ff, $000007ff,
    $00000fff, $00001fff, $00003fff, $00007fff,
    $0000ffff, $0001ffff, $0003ffff, $0007ffff,
    $000fffff, $001fffff, $003fffff, $007fffff,
    $00ffffff, $01ffffff, $03ffffff, $07ffffff,
    $0fffffff, $1fffffff, $3fffffff, $7fffffff,
    $ffffffff );

  bit_shift: array[0..39] of ULONG = (
    $00000001, $00000002, $00000004, $00000008,
    $00000010, $00000020, $00000040, $00000080,
    $00000100, $00000200, $00000400, $00000800,
    $00001000, $00002000, $00004000, $00008000,
    $00010000, $00020000, $00040000, $00080000,
    $00100000, $00200000, $00400000, $00800000,
    $01000000, $02000000, $04000000, $08000000,
    $10000000, $20000000, $40000000, $80000000,
    $80000000, $80000000, $80000000, $80000000,
    $80000000, $80000000, $80000000, $80000000 );

  shift_16: array[0..35] of ULONG = (
    $00000010, $00000020, $00000040, $00000080,
    $00000100, $00000200, $00000400, $00000800,
    $00001000, $00002000, $00004000, $00008000,
    $00010000, $00020000, $00040000, $00080000,
    $00100000, $00200000, $00400000, $00800000,
    $01000000, $02000000, $04000000, $08000000,
    $10000000, $20000000, $40000000, $80000000,
    $80000000, $80000000, $80000000, $80000000,
    $80000000, $80000000, $80000000, $80000000 );

  flt_set: array[0..3,0..1] of LongInt = (
    (10,1), (9,1), (10,1), (12,0)
  );

  procedure tta_encoder_init(tta: Pencoder; nch: LongInt; byte_size: LongInt);

implementation

(*procedure tta_rice_init(var rice: Tadapt; k0: ULONG; k1: ULONG);
begin
  rice.k0 := k0;
  rice.k1 := k1;
  rice.sum0 := shift_16[k0];
  rice.sum1 := shift_16[k1];
end;*)

procedure tta_encoder_init(tta: Pencoder; nch: LongInt; byte_size: LongInt);
var
  fset0, fset1: LongInt;
  i: integer;
begin
  fset0 := flt_set[byte_size - 1,0];
  fset1 := flt_set[byte_size - 1,1];
  
  for i := 0 to nch-1 do begin
//  filter_init(@tta[i].fst, fset0, fset1);
    fillchar(tta.fst, sizeof(Tfltst), 0);
    tta.fst.shift := fset0;
    tta.fst.round := 1 shl (fset0 - 1);
    tta.fst.mutex := fset1;
//  tta_rice_init(tta.rice, 10, 10);
    tta.rice.k0 := 10;
    tta.rice.k1 := 10;
    tta.rice.sum0 := shift_16[10];
    tta.rice.sum1 := shift_16[10];
// end of tta_rice_init
    tta.last := 0;
    inc(tta);
  end;
end;

end.
